﻿using DotNetCore.CAP;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Mvc;
using Sharing.Application;
using Sharing.Notifications.HuiJu;
using Sharing.Notifications.XieTong;
using Sharing.Orders;
using Sharing.Province.Dtos.HuiJu.Send;
using Sharing.Province.Dtos.XieTong.Send;
using Sharing.Province.HuiJu.Send;
using Sharing.Province.XieTong.Send;

namespace Sharing.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class HotlineMessageReceiveController : Controller
    {
        #region 注入
        private readonly IDataOrderRepository _dataOrderRepository;
        private readonly IMediator _mediator;
        private readonly ILogger<HotlineMsgReceiver> _logger;
        private readonly IMapper _mapper;
        private readonly IGetCaseResultReceiveRepository _getCaseResultReceiveRepository;
        private readonly IGetCaseProcessReceiveRepository _getCaseProcessReceiveRepository;
        private readonly IGetVisitInfoReceiveRepository _getVisitInfoReceiveRepository;
        private readonly IGetCaseBackApplyRepository _getCaseBackApplyRepository;
        private readonly IDelayCaseInfoSendRepository _delayCaseInfoSendRepository;
        private readonly ISendSuperviseProcessInfoRepository _sendSuperviseProcessInfoRepository;
        private readonly ISendSuperviseResultInfoRepository _sendSuperviseResultInfoRepository;
        private readonly ISubmitCaseInfoRepository _submitCaseInfoRepository;
        private readonly ISubmitCaseResultRepository _submitCaseResultRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="dataOrderRepository"></param>
        /// <param name="mediator"></param>
        /// <param name="logger"></param>
        /// <param name="mapper"></param>
        /// <param name="getCaseResultReceiveRepository"></param>
        /// <param name="getCaseProcessReceiveRepository"></param>
        /// <param name="getVisitInfoReceiveRepository"></param>
        /// <param name="getCaseBackApplyRepository"></param>
        /// <param name="delayCaseInfoSendRepository"></param>
        /// <param name="sendSuperviseProcessInfoRepository"></param>
        /// <param name="sendSuperviseResultInfoRepository"></param>
        /// <param name="submitCaseInfoRepository"></param>
        public HotlineMessageReceiveController(IDataOrderRepository dataOrderRepository, IMediator mediator, ILogger<HotlineMsgReceiver> logger, IMapper mapper
            , IGetCaseResultReceiveRepository getCaseResultReceiveRepository, IGetCaseProcessReceiveRepository getCaseProcessReceiveRepository
            , IGetVisitInfoReceiveRepository getVisitInfoReceiveRepository, IGetCaseBackApplyRepository getCaseBackApplyRepository
            , IDelayCaseInfoSendRepository delayCaseInfoSendRepository, ISendSuperviseProcessInfoRepository sendSuperviseProcessInfoRepository
            , ISendSuperviseResultInfoRepository sendSuperviseResultInfoRepository, ISubmitCaseInfoRepository submitCaseInfoRepository
            , ISubmitCaseResultRepository submitCaseResultRepository)
        {
            _dataOrderRepository = dataOrderRepository;
            _mediator = mediator;
            _logger = logger;
            _mapper = mapper;
            _getCaseResultReceiveRepository = getCaseResultReceiveRepository;
            _getCaseProcessReceiveRepository = getCaseProcessReceiveRepository;
            _getVisitInfoReceiveRepository = getVisitInfoReceiveRepository;
            _getCaseBackApplyRepository = getCaseBackApplyRepository;
            _delayCaseInfoSendRepository = delayCaseInfoSendRepository;
            _sendSuperviseProcessInfoRepository = sendSuperviseProcessInfoRepository;
            _sendSuperviseResultInfoRepository = sendSuperviseResultInfoRepository;
            _submitCaseInfoRepository = submitCaseInfoRepository;
            _submitCaseResultRepository = submitCaseResultRepository;
        }
        #endregion

        #region 协同-第一批次
        /// <summary>
        /// 服务工单交办处理
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task GetCaseResultReceive(GetCaseResultReceiveInfo dto)
        {
            var data = _mapper.Map<GetCaseResultReceive>(dto);

            //将上报信息写入本地库
            var id = await _getCaseResultReceiveRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new GetCaseResultReceiveNotification(data));

        }

        /// <summary>
        /// 服务工单交办处理过程  
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task GetCaseProcessReceive(GetCaseProcessReceiveInfo dto)
        {
            var data = _mapper.Map<GetCaseProcessReceive>(dto);

            //将上报信息写入本地库
            var id = await _getCaseProcessReceiveRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new GetCaseProcessReceiveNotification(data));
        }

        /// <summary>
        /// 服务工单交办评价  
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task GetVisitInfoReceive(GetVisitInfoReceiveInfo dto)
        {
            var data = _mapper.Map<GetVisitInfoReceive>(dto);

            //将上报信息写入本地库
            var id = await _getVisitInfoReceiveRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new GetVisitInfoReceiveNotification(data));
        }

        /// <summary>
        /// 退回申请  
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task GetCaseBackApply(GetCaseBackApplyInfo dto)
        {
            var data = _mapper.Map<GetCaseBackApply>(dto);

            //将上报信息写入本地库
            var id = await _getCaseBackApplyRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new GetCaseBackApplyNotification(data));
        }

        /// <summary>
        /// 服务工单申请延时  
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task DelayCaseInfoSend(DelayCaseInfoSendInfo dto)
        {
            var data = _mapper.Map<DelayCaseInfoSend>(dto);

            //将上报信息写入本地库
            var id = await _delayCaseInfoSendRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new DelayCaseInfoSendNotification(data));
        }

        /// <summary>
        /// 服务工单督办过程  
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task SendSuperviseProcessInfo(SendSuperviseProcessInfoInfo dto)
        {
            var data = _mapper.Map<SendSuperviseProcessInfo>(dto);

            //将上报信息写入本地库
            var id = await _sendSuperviseProcessInfoRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new SendSuperviseProcessInfoNotification(data));
        }

        /// <summary>
        /// 服务工单督办结果  
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task SendSuperviseResultInfo(SendSuperviseResultInfoInfo dto)
        {
            var data = _mapper.Map<SendSuperviseResultInfo>(dto);

            //将上报信息写入本地库
            var id = await _sendSuperviseResultInfoRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new SendSuperviseResultInfoNotification(data));
        }

        #endregion

        /// <summary>
        /// 服务工单受理  --- 部分字段未赋值处理
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        [CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task ReceiveOrderCreated(OrderDto dto)
        {
            //处理数据
            SubmitCaseInfo submitCaseInfo = new SubmitCaseInfo
            {
                OrderID = dto.Id,
                No = dto.No,
                CaseTitle = dto.Title,
                CaseRegister = dto.EmployeeName,
                CaseRegisterNo = dto.EmployeeStaffNo,
                CaseDate = Convert.ToDateTime(dto.CreationTime),
                PhoneNumber = dto.Contact,
                AppLicantName = dto.FromName,
                AppLicantAge = dto.AgeRangeCode,
                AppLicantId = dto.LicenceNo,
                CaseAccord = "",
                CaseContent = dto.Content,
                CaseProcessType = "",
                CaseIsPublic = "",
                CaseIsSecrect = "",
                CaseIsVisit = "",
                CaseValid = "",
                TSigntTmeBf = null,
                TfdBackTimeBf = null,
                TBackTimeBf = null,
                RelateSerial = "",
                CaseLngLat = "",
                CaseEmotion = "",
                CaseAddress = "",
                EventDate = null,
                CaseGoal = "",
                ThemeSerial = "",
                AdviseType = "",
                AdviseDept = "",
                ComplainType = "",
                ComplainDept = "",
                PushType = "",

            };

            //处理诉求渠道
            switch (dto.Channel)
            {
                case EChannel.Phone:
                    submitCaseInfo.CaseSource = "RGDH";
                    break;
                case EChannel.WxOfficial:
                    submitCaseInfo.CaseSource = "WX";
                    break;
                case EChannel.Web:
                    submitCaseInfo.CaseSource = "WZ";
                    break;
                case EChannel.App:
                    submitCaseInfo.CaseSource = "Ap";
                    break;
                case EChannel.WxMiniProgram:
                    submitCaseInfo.CaseSource = "XCX";
                    break;
                default:
                    break;
            }

            //处理性别
            switch (dto.FromGender)
            {
                case EGender.Female:
                    submitCaseInfo.AppLicantSex = "0";
                    break;
                case EGender.Male:
                    submitCaseInfo.AppLicantSex = "1";
                    break;
                case EGender.Unknown:
                    submitCaseInfo.AppLicantSex = "2";
                    break;
                default:
                    break;
            }

            //处理证件类型
            switch (dto.LicenceTypeCode)
            {
                case "10":
                    submitCaseInfo.CertifiCateType = "10";
                    break;
                case "40":
                    submitCaseInfo.CertifiCateType = "20";
                    break;
                case "55":
                    submitCaseInfo.CertifiCateType = "30";
                    break;
                case "57":
                    submitCaseInfo.CertifiCateType = "40";
                    break;
                default:
                    submitCaseInfo.CertifiCateType = "50";
                    break;
            }

            //处理受理类型
            switch (dto.AcceptType)
            {
                case EAcceptType.Consult:
                    submitCaseInfo.CaseType = "10";
                    break;
                case EAcceptType.Suggest:
                    submitCaseInfo.CaseType = "15";
                    break;
                case EAcceptType.SeekHelp:
                    submitCaseInfo.CaseType = "20";
                    break;
                case EAcceptType.Praise:
                    submitCaseInfo.CaseType = "25";
                    break;
                case EAcceptType.Report:
                    submitCaseInfo.CaseType = "30";
                    break;
                case EAcceptType.Complain:
                    submitCaseInfo.CaseType = "35";
                    break;
                case EAcceptType.Other:
                    submitCaseInfo.CaseType = "40";
                    break;
                default:
                    break;
            }

            //处理紧急程度
            switch (dto.EmergencyLevel)
            {
                case EEmergencyLevel.QuiteEmergency:
                    submitCaseInfo.CaseIsUrgent = "10";
                    break;
                case EEmergencyLevel.Emergency:
                    submitCaseInfo.CaseIsUrgent = "20";
                    break;
                default:
                    submitCaseInfo.CaseIsUrgent = "30";
                    break;
            }





            submitCaseInfo.Id = await _submitCaseInfoRepository.AddAsync(submitCaseInfo, HttpContext.RequestAborted);

            await _mediator.Publish(new OrderCreatedNotification(submitCaseInfo));
        }

        /// <summary>
        /// 服务工单处理结果
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [NonAction]
        //[CapSubscribe(Hotline.Share.Mq.EventNames.HotlineOrderCreated)]
        public async Task SubmitCaseResult(CaseResult dto)
        {
            var data = _mapper.Map<SubmitCaseResult>(dto);

            //将上报信息写入本地库
            var id = await _submitCaseResultRepository.AddAsync(data);

            data.Id = id;
            //信息上报
            await _mediator.Publish(new SubmitCaseResultNotification(data));

        }
    }
}
